@@ -16,7 +16,7 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
- * $Id: pa_bitstring.ml 179 2010-01-08 14:28:17Z richard.wm.jones $
+ * $Id: pa_bitstring.ml 189 2012-01-17 13:02:18Z richard.wm.jones@gmail.com $
  *)
 
 open Printf
@@ -989,7 +989,8 @@ let output_bitmatch _loc bs cases =
 	match bind with
 	| Some name ->
 	    <:expr<
-	      let $lid:name$ = ($lid:data$, $lid:off$, $lid:len$) in
+	      let $lid:name$ = ($lid:data$,
+                                $lid:original_off$, $lid:original_len$) in
 	      $inner$
 	      >>
 	| None -> inner in
